/* This file is part of the KDE project
   Copyright (C) 2001 Eva Brucherseifer <eva@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <iostream.h>


#include <qlayout.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qlineedit.h>
#include <qstylesheet.h>
#include <qtextbrowser.h>
#include <qcolor.h>
#include <qcombobox.h>


#include <kglobal.h>
#include <klocale.h>
#include <kstddirs.h>
#include <kconfig.h>
#include <kapp.h>
#include <kcolorbutton.h>
#include <kfontdialog.h>
#include <kurlrequester.h>

#include "kvcardconfig.h"

#include "kvcardconfigbase.h"


KvcardConfig::KvcardConfig(QWidget *parent, const char *name, bool modal)
		: QDialog(parent, name, modal)
{
	dialog = new KvcardConfigBase(this);
	dialog->show();

	modified=false;

	QStringList fonts;
	KFontChooser::getFontList(fonts, false);
	dialog->fontFamily->insertStringList(fonts);

	connect(dialog->basefontsize, SIGNAL(highlighted(int)),
	        this, SLOT(configChanged()));
	connect(dialog->basefontsize, SIGNAL(textChanged(const QString&)),
	        this, SLOT(configChanged()));
	connect(dialog->blackOnWhite, SIGNAL(clicked()),
	        this, SLOT(configChanged()));
	connect(dialog->whiteOnBlack, SIGNAL(clicked()),
	        this, SLOT(configChanged()));
	connect(dialog->customColor, SIGNAL(clicked()),
	        this, SLOT(configChanged()));
	connect(dialog->foregroundColor, SIGNAL(changed(const QColor &)),
	        this, SLOT(configChanged()));
	connect(dialog->backgroundColor, SIGNAL(changed(const QColor &)),
	        this, SLOT(configChanged()));
	connect(dialog->linkColor, SIGNAL(changed(const QColor &)),
	        this, SLOT(configChanged()));
	connect(dialog->fontFamily, SIGNAL(highlighted(int)),
	        this, SLOT(configChanged()));
	connect(dialog->fontFamily, SIGNAL(textChanged(const QString&)),
	        this, SLOT(configChanged()));
	connect(dialog->sameColor, SIGNAL(clicked()),
	        this, SLOT(configChanged()));

	connect(dialog->buttonDefaults, SIGNAL(clicked()),
	        this, SLOT(defaults()));
	connect(dialog->buttonOK, SIGNAL(clicked()),
	        this, SLOT(accept()));
	connect(dialog->buttonCancel, SIGNAL(clicked()),
	        this, SLOT(reject()));

	QVBoxLayout *vbox = new QVBoxLayout(this, 0, 0);
	vbox->addWidget(dialog);

	load();
}


void KvcardConfig::load()
{
	KConfig *c = new KConfig("kvcardpartrc", false, false);

	c->setGroup("Font");
	dialog->basefontsize->setEditText(QString::number(c->readNumEntry("BaseSize", 12)));

	QString fname = c->readEntry("Family", "Arial");
	for (int i=0; i < dialog->fontFamily->count(); ++i)
		if (dialog->fontFamily->text(i) == fname)
		{
			dialog->fontFamily->setCurrentItem(i);
			break;
		}

	c->setGroup("Colors");
	QString m = c->readEntry("Mode", "black-on-white");
	dialog->blackOnWhite->setChecked(m == "black-on-white");
	dialog->whiteOnBlack->setChecked(m == "white-on-black");
	dialog->customColor->setChecked(m == "custom");
	dialog->backgroundColor->setColor(c->readColorEntry("BackColor", &Qt::white));
	dialog->foregroundColor->setColor(c->readColorEntry("ForeColor", &Qt::black));
	dialog->linkColor->setColor(c->readColorEntry("LinkColor", &Qt::blue));
	dialog->sameColor->setChecked(c->readBoolEntry("SameColor", false));

	delete c;
}


void KvcardConfig::save()
{
	// write to config file
	KConfig *c = new KConfig("kvcardpartrc", false, false);

	c->setGroup("Font");
	c->writeEntry("BaseSize", dialog->basefontsize->currentText());
	c->writeEntry("Family", dialog->fontFamily->currentText());

	c->setGroup("Colors");
	if (dialog->blackOnWhite->isChecked())
		c->writeEntry("Mode", "black-on-white");
	if (dialog->whiteOnBlack->isChecked())
		c->writeEntry("Mode", "white-on-black");
	if (dialog->customColor->isChecked())
		c->writeEntry("Mode", "custom");
	c->writeEntry("BackColor", dialog->backgroundColor->color());
	c->writeEntry("ForeColor", dialog->foregroundColor->color());
	c->writeEntry("LinkColor", dialog->linkColor->color());
	c->writeEntry("SameColor", dialog->sameColor->isChecked());

	c->sync();
	delete c;
}


void KvcardConfig::defaults()
{
	dialog->basefontsize->setEditText(QString::number(12));

	QString fname =  "Arial";
	for (int i=0; i < dialog->fontFamily->count(); ++i)
		if (dialog->fontFamily->text(i) == fname)
		{
			dialog->fontFamily->setCurrentItem(i);
			break;
		}

	dialog->blackOnWhite->setChecked(true);
	dialog->whiteOnBlack->setChecked(false);
	dialog->customColor->setChecked(false);
	dialog->backgroundColor->setColor(Qt::white);
	dialog->foregroundColor->setColor(Qt::black);
	dialog->linkColor->setColor(Qt::blue);
	dialog->sameColor->setChecked(false);
}


void KvcardConfig::configChanged()
{
	modified=true;
}


QString px(int i, double scale)
{
	QString px;
	px.setNum(static_cast<int>(i * scale));
	px += "px";
	return px;
}


