/* This file is part of the KDE project
   Copyright (C) 2000 Eva Brucherseifer <eva@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <qvaluelist.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qdatetime.h>
#include <qvariant.h>
#include <qdialog.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qtoolbutton.h>
#include <qwidgetstack.h>
#include <qlistbox.h>
#include <qheader.h>
#include <qcombobox.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qmultilineedit.h>

#include "kdebug.h"
#include "klocale.h"
#include "kdatepik.h"
#include "kiconloader.h"
#include "kglobal.h"


#include "vcard.h"
#include "kvcardeditor.h"
#include "kvcardeditoraddress.h"




KDateDialog::KDateDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
		: QDialog( parent, name, modal, fl )
{
	if ( !name )
		setName( "KDateDialog" );

	resize( 200, 300 );
	setCaption( i18n( "Edit Date" ) );
	dialogLayout = new QGridLayout( this );
	dialogLayout->setSpacing( 3 );
	dialogLayout->setMargin( 6 );

	picker = new KDatePicker( this );
	dialogLayout->addWidget( picker, 0, 0 );

	QHBoxLayout* buttonsLayout = new QHBoxLayout(this);
	buttonsLayout->setSpacing( 6 );
	buttonsLayout->setMargin( 0 );
	dialogLayout->addLayout( buttonsLayout, 1, 0);

	buttonOK = new QPushButton( i18n("&OK"), this, "ok");
	buttonsLayout->addWidget( buttonOK);
	buttonsLayout->insertStretch(-1, 2);

	buttonCancel = new QPushButton( i18n("&Cancel"), this, "cancel");
	buttonsLayout->addWidget( buttonCancel);

	// signals and slots connections
	connect( picker, SIGNAL( dateEntered(QDate) ), this, SLOT( accept() ) );
	connect( buttonOK, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
}


/* ****************************************************************/

KvcardEditor::KvcardEditor( QWidget* parent, const char* name, bool modal )
		: KvcardEditorBase( parent, name, modal)
{
	m_vcard = new VCard();
	m_addressCount = 0;

	stackAddress->show();
	listAddresses->setSelected(0, true);
	buttonBirthday->setIconSet( QIconSet( KGlobal::iconLoader()->loadIcon("date.xpm", KIcon::Small) ) );
}

void KvcardEditor::setVCard( VCard* vcard )
{
	if (m_vcard)
		delete m_vcard;
	m_vcard = new VCard(*vcard);
	displayVCard();
};

KvcardEditor::~KvcardEditor()
{
	if (m_vcard)
		delete m_vcard;
}


void KvcardEditor::slotApply()
{
	kdDebug() << "KvcardEditor::slotApply()" << endl;
	if (m_vcard)
		readVCard();
	return;
}

void KvcardEditor::slotOK()
{
	kdDebug() << "KvcardEditor::slotOK()" << endl;
	if (m_vcard)
	{
		readVCard();
		accept();
		return;
	}
	kdDebug() << "KvcardEditor::slotOK() - no vcard set" << endl;
	reject();
	return;
}

	


//--------------------------------------------------------------------------------


void KvcardEditor::displayVCard()
{
	kdDebug() << "KvcardEditor::displayVCard()" << endl;

	editFormattedName->setText(m_vcard->getValue(VCARD_FN));

	QValueList<QString> name = m_vcard->getValues(VCARD_N);
	kdDebug() << "name count: " << name.count() << endl;
	switch(name.count())
	{
	case 4:
		editNameSuffix->setText(name[3]);
	case 3:
		editNamePrefix->setText(name[2]);
	case 2:
		editGivenName->setText(name[1]);
	case 1:
		editFamilyName->setText(name[0]);
	}

	editNickName->setText(m_vcard->getValue(VCARD_NICKNAME));

	QValueList<VCardLine> lines;

	addressViews.clear();
	
	lines = m_vcard->getLines(VCARD_ADR);
	kdDebug() << "KvcardEditor::displayVCard(): getting " << lines.count() << " lines of type " << VCARD_ADR << endl;
	
	for (QValueList<VCardLine>::Iterator lineIter=lines.begin() ; lineIter!=lines.end() ; lineIter++)
		displayAddress(*lineIter);
	if (lines.count()>0)
		listAddresses->setCurrentItem(0);

	QString bday = m_vcard->getValue(VCARD_BDAY);
	if (!bday.isEmpty())
	{
		QString day = QString(bday[6]) + bday[7];
		QString month = QString(bday[4]) + bday[5];
		QString year = bday;
		year.truncate(4);
		textBirthday->setText( day + "." + month + "." + year );
	}
	else
		textBirthday->setText( "" );
		
	editNote->setText(m_vcard->getValue(VCARD_NOTE));
	editRole->setText(m_vcard->getValue(VCARD_ROLE));
	editAgent->setText(m_vcard->getValue(VCARD_AGENT));
	editTitle->setText(m_vcard->getValue(VCARD_TITLE));
	
	QValueList<QString> org = m_vcard->getValues(VCARD_ORG);
	QValueList<QString>::ConstIterator orgIter;
	QString orgstring;
	for (orgIter=org.begin(); orgIter!=org.end(); orgIter++)
			orgstring += *orgIter + "\n";
	editOrg->setText(orgstring);
			
	return;
}



void KvcardEditor::readVCard()
{
	kdDebug() << "KvcardEditor::readVCard()" << endl;
	QString value;
	//m_vcard->clean();

	// read name
	if ( ! (editFormattedName->text()).isEmpty() )
	{
		m_vcard->removeLine(VCARD_FN);
		m_vcard->addLine(VCARD_FN, editFormattedName->text());
	}

	if ( ! ( (editFamilyName->text()).isEmpty() && (editGivenName->text()).isEmpty()
		&& (editNamePrefix->text()).isEmpty() && (editNameSuffix->text()).isEmpty() ) )
	{
		VCardLine::ValueList name;
		m_vcard->removeLine(VCARD_N);
		name.append(editFamilyName->text());
		name.append(editGivenName->text());
		name.append(editNamePrefix->text());
		name.append(editNameSuffix->text());
		m_vcard->addLine(VCARD_N, name);
	}

	if ( ! (editNickName->text()).isEmpty() )
	{
		m_vcard->removeLine(VCARD_NICKNAME);
		m_vcard->addLine(VCARD_NICKNAME, editNickName->text());
	}
		
	// read addresses
	while(m_vcard->hasValue(VCARD_ADR))
		m_vcard->removeLine(VCARD_ADR);
		
	QValueList<KvcardAddress*>::Iterator addrIter;
	for ( addrIter = addressViews.begin();
        	addrIter != addressViews.end(); addrIter++ )
	{
		readAddress(*addrIter);
	}

	value = textBirthday->text();
	if (!value.isEmpty())
	{
		m_vcard->removeLine(VCARD_BDAY);
		m_vcard->addLine(VCARD_BDAY, value);
	}

	value = editTitle->text();
	if (!value.isEmpty())
	{
		m_vcard->removeLine(VCARD_TITLE);
		m_vcard->addLine(VCARD_TITLE, value);
	}

	value = editRole->text();
	if (!value.isEmpty())
	{
		m_vcard->removeLine(VCARD_ROLE);
		m_vcard->addLine(VCARD_ROLE, value);
	}
	
	value = editAgent->text();
	if (!value.isEmpty())
	{
		m_vcard->removeLine(VCARD_AGENT);
		m_vcard->addLine(VCARD_AGENT, value);
	}

	value = editOrg->text();
	if (!value.isEmpty())
	{
		QValueList<QString> values;
		int times, i, lastpos, pos;
		QString part;
		times = value.contains("\n");
		lastpos=0;
		for (i=0; i<times; i++)
		{
			pos=value.find("\n");
			part = value.mid(lastpos+1, pos);
			values.append(part);
			lastpos=pos;
		}
		m_vcard->removeLine(VCARD_ORG);
		m_vcard->addLine(VCARD_ORG, value);
	}

	value = editNote->text();
	if (!value.isEmpty())
	{
		m_vcard->removeLine(VCARD_NOTE);
		m_vcard->addLine(VCARD_NOTE, value);
	}

	
	kdDebug() << m_vcard->getVCard() << endl;
}


bool KvcardEditor::displayAddress(VCardLine& addressline)
{
	QValueList<QString> valuelist;
	QValueList<QString> qualifiers;

	// get data
	valuelist = addressline.getValues();
	qualifiers = addressline.getTypes();
	
	kdDebug() << "KvcardEditor::displayAddress " << valuelist.count() << endl;

	// display data
	if(valuelist.count()>0)
	{
		KvcardAddress* addressview = new KvcardAddress(this);

		// display the address
		addressview->editPOBox->setText(valuelist[0]);
		// addressview->editExtendedAddress->setText(valuelist[1]);
		addressview->editStreet->setText(valuelist[2]);
		addressview->editCity->setText(valuelist[3]);
		addressview->editRegion->setText(valuelist[4]);
		addressview->editZIP->setText(valuelist[5]);
		addressview->editCountry->setText(valuelist[6]);

		addressview->radioHome->setChecked(addressline.hasQualifierOrType(VCARD_ADR_HOME));
		addressview->radioWork->setChecked(addressline.hasQualifierOrType(VCARD_ADR_WORK));
		
		addressview->checkDom->setChecked(addressline.hasQualifierOrType(VCARD_ADR_DOM));
		addressview->checkIntl->setChecked(addressline.hasQualifierOrType(VCARD_ADR_INTL));
		addressview->checkPostal->setChecked(addressline.hasQualifierOrType(VCARD_ADR_POSTAL));
		addressview->checkParcel->setChecked(addressline.hasQualifierOrType(VCARD_ADR_PARCEL));
		
		addressview->radioPref->setChecked(addressline.hasQualifierOrType(VCARD_ADR_PREF));

		stackAddress->addWidget(addressview, m_addressCount);
		connect(addressview->radioPref, SIGNAL(toggled(bool)), this, SLOT(slotPrefAddress(bool)));

		QString itemTitle = i18n("Address %1").arg(m_addressCount+1);
		listAddresses->insertItem(itemTitle, m_addressCount);
		addressViews.append(addressview);
		m_addressCount++;

		return true;
	}
	return false;
}


bool KvcardEditor::readAddress(KvcardAddress* addressview)
{
	QValueList<QString> addressTypes;
	QValueList<QString> addressQualifier;
	QValueList<QString> addresslist;
	bool typed=false;

	if (addressview->radioHome->isChecked())
	{
		addressTypes.append(QString(VCARD_ADR_HOME));
		typed=true;
	}
	if (addressview->radioWork->isChecked())
	{
		addressTypes.append(QString(VCARD_ADR_WORK));
		typed=true;
	}
	
	if (typed==true)
	{
		if (addressview->checkDom->isChecked()) addressTypes.append(QString(VCARD_ADR_DOM));
		if (addressview->checkIntl->isChecked()) addressTypes.append(QString(VCARD_ADR_INTL));
		if (addressview->checkPostal->isChecked()) addressTypes.append(QString(VCARD_ADR_POSTAL));
		if (addressview->checkParcel->isChecked()) addressTypes.append(QString(VCARD_ADR_PARCEL));
		if (addressview->radioPref->isChecked()) addressTypes.append(QString(VCARD_ADR_PREF));
	}

	if ( ! ( (addressview->editPOBox->text()).isEmpty() && (addressview->editStreet->text()).isEmpty()
		&& (addressview->editCity->text()).isEmpty() && (addressview->editRegion->text()).isEmpty()
		&& (addressview->editZIP->text()).isEmpty() && (addressview->editCountry->text()).isEmpty()
		) )
	{
		addresslist.append(addressview->editPOBox->text());
		addresslist.append(QString(""));    // extended
		addresslist.append(addressview->editStreet->text());
		addresslist.append(addressview->editCity->text());
		addresslist.append(addressview->editRegion->text());
		addresslist.append(addressview->editZIP->text());
		addresslist.append(addressview->editCountry->text());

		m_vcard->addLine(VCARD_ADR, addresslist, addressTypes, addressQualifier);

		return true;
	}
	return false;
}


void KvcardEditor::slotPrefAddress(bool toggled)
{
	kdDebug()  << "KvcardEditor::slotPrefAddress()" << endl;
	
	QWidget* prefAddressWidget = stackAddress->visibleWidget();
	QValueList<KvcardAddress*>::Iterator addressIter;
	
	for (addressIter=addressViews.begin(); addressIter!=addressViews.end(); addressIter++)
	{
		if (*addressIter!=prefAddressWidget)
			(*addressIter)->radioPref->setChecked(false);
	}
	return;
}


void KvcardEditor::slotNewAddress()
{
		KvcardAddress* addressview = new KvcardAddress(this);
		stackAddress->addWidget(addressview, m_addressCount);

		QString itemTitle = i18n("Address %1").arg(m_addressCount+1);
		listAddresses->insertItem(itemTitle, m_addressCount);
		addressViews.append(addressview);
		m_addressCount++;
}


void KvcardEditor::slotDeleteAddress()
{
	QWidget* removedAddress = stackAddress->visibleWidget();
	listAddresses->removeItem(listAddresses->currentItem());
	stackAddress->removeWidget(removedAddress);
}


void KvcardEditor::slotBirthday()
{
	QDate birthday;
	KDateDialog* date = new KDateDialog(this, "date", true);
	QString bday = m_vcard->getValue(VCARD_BDAY);
	if (!bday.isEmpty())
	{
		QString day = QString(bday[6]) + bday[7];
		QString month = QString(bday[4]) + bday[5];
		QString year = bday;
		year.truncate(4);
		date->picker->setDate( QDate( year.toInt() , month.toInt() , day.toInt() ) );
	}

	if (date->exec()==QDialog::Accepted)
	{
		m_vcard->removeLine(VCARD_BDAY);
		birthday = date->picker->getDate();
		m_vcard->addLine(VCARD_BDAY, birthday.toString());
		QString day, month, year;
		day.setNum( birthday.day() );
		month.setNum( birthday.month() );
		year.setNum( birthday.year() );

		textBirthday->setText( day + "." + month + "." + year );
	}
	return;
}


