/* This file is part of the KDE project
   Copyright (C) 2000 Eva Brucherseifer <eva@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <qtextview.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qtable.h>
#include <qlistview.h>

#include <kapp.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kconfig.h>

#include "vcard.h"
#include "kvcardlayout.h"



const QString defaultstring=i18n("Default");
const QString workstring=i18n("Work");
const QString homestring=i18n("Home");
const QString domstring=i18n("Domestic");
const QString intlstring=i18n("International");
const QString postalstring=i18n("Postal");
const QString parcelstring=i18n("Parcel");
const QString prefstring=i18n("Preferred");

KvcardLayout::KvcardLayout( QTextView* view)
		: QObject( )
{
	m_view = view;
	m_vcard=0L;

	syncPreferences();
}

/*
 *  Destroys the object and frees any allocated resources
 */
KvcardLayout::~KvcardLayout()
{
	// no need to delete child widgets, Qt does it all for us
}

void KvcardLayout::syncPreferences()
{
	KConfig *c = new KConfig("kvcardpartrc", false, false);

	c->setGroup("Font");
	fontSize = c->readNumEntry("BaseSize", 12);
	fontName = c->readEntry("Family", "Arial");

	c->setGroup("Colors");
	colorMode = c->readEntry("Mode", "black-on-white");

	backgroundColor=c->readColorEntry("BackColor", &Qt::white);
	foregroundColor=c->readColorEntry("ForeColor", &Qt::black);
	linkColor=Qt::blue;
	if (colorMode=="black-on-white")
	{
		backgroundColor=Qt::white;
		foregroundColor=Qt::black;
		linkColor=Qt::blue;
	}
	if (colorMode=="white-on-black")
	{
		backgroundColor=Qt::black;
		foregroundColor=Qt::white;
		linkColor=Qt::yellow;
	}		
	sameColor=c->readBoolEntry("SameColor", false);
	if (sameColor)
		linkColor=foregroundColor;
	delete c;
	
	QColorGroup colorlayout;
	colorlayout.setColor(QColorGroup::Text,foregroundColor);
	colorlayout.setColor(QColorGroup::Base, backgroundColor);
	m_view->setPaperColorGroup(colorlayout);
	m_view->setLinkColor(linkColor);

	m_view->setFont(QFont(fontName, fontSize));
}


void KvcardLayout::displayVCard(VCard* newcard)
{
	if (!newcard)
		return;
	m_vcard = newcard;

	int i;
	QString vcardstring = "";
	QString value;
	QValueList<QString> values;
	QValueList<VCardLine> lines;
	QValueList<VCardLine>::ConstIterator linesIter;
	QString tmpstr;
	
	// header and name
	vcardstring =  "<qt>\n";
	vcardstring += "<h3>";
	insertIdentity(vcardstring);
	vcardstring += "</h3>";

	// Identification
	QString identification;
	insertTableValues(identification, i18n("Organization"),  VCARD_ORG);
	insertTableValues(identification, i18n("Title"),  VCARD_TITLE);
	insertTableValues(identification, i18n("Role"), VCARD_ROLE );

	lines = m_vcard->getLines(VCARD_EMAIL);
	for (linesIter=lines.begin();linesIter!=lines.end();linesIter++)
	{
		value=(*linesIter).getValue();
		if (!value.isEmpty())
		{
			identification += "<tr><td><i>"+ i18n("Email") + "</i></td>";
			identification += "<td><a href=\"" + value + "\">" + value + "</a></td>";
			identification += "</td></tr>\n";
		}
	}

	value = m_vcard->getValue(VCARD_URL);
	if (!value.isEmpty())
	{
		identification += "<tr><td><i>" + i18n("URL") + "</i></td>";
		identification += "<td><a href=\"" + value + "/index.html\">" + value + "</a></td>";
		identification += "</td></tr>\n";
	}

	insertTableValue(identification, i18n("Birthday"),  VCARD_BDAY);

	if (!identification.isEmpty())
	{
		vcardstring += "<table>\n";
		vcardstring += identification;
		vcardstring += "</table>\n";
		vcardstring += "<br/>\n";
	}

	// Delivery
	QString addresses;
	/*
	insertTableValues(addresses, i18n("Default"), VCARD_ADR);
	insertTableValues(addresses, i18n("Work"),  VCARD_ADR,VCARD_ADR_WORK);
	insertTableValues(addresses, i18n("Home"),  VCARD_ADR,VCARD_ADR_HOME);
	insertTableValues(addresses, i18n("Domestic"),  VCARD_ADR, VCARD_ADR_DOM);
	insertTableValues(addresses, i18n("International"),  VCARD_ADR, VCARD_ADR_INTL);
	insertTableValues(addresses, i18n("Postal"),  VCARD_ADR, VCARD_ADR_POSTAL);
	insertTableValues(addresses, i18n("Parcel"),  VCARD_ADR, VCARD_ADR_PARCEL);
	insertTableValues(addresses, i18n("Preferred"),  VCARD_ADR, VCARD_ADR_PREF);
	*/
	insertAddresses(addresses);	
	
	if (!addresses.isEmpty())
	{
		vcardstring += "<b>" + i18n("Addresses") + ":</b><br/>\n";
		vcardstring += "<table>\n";
		vcardstring += addresses;
		vcardstring += "</table>\n";
		vcardstring += "<br/>\n";
	}

	// Phone Numbers
	QString phonenumbers;
	insertTableValue(phonenumbers,  i18n("Home"), VCARD_TEL,  VCARD_TEL_HOME);
	insertTableValue(phonenumbers, i18n("Work"), VCARD_TEL,  VCARD_TEL_WORK);
	insertTableValue(phonenumbers, i18n("Preferred"), VCARD_TEL,  VCARD_TEL_PREF);
	insertTableValue(phonenumbers, i18n("Voice"), VCARD_TEL,  VCARD_TEL_VOICE);
	insertTableValue(phonenumbers, i18n("Fax"), VCARD_TEL,  VCARD_TEL_FAX);
	insertTableValue(phonenumbers, i18n("Message"), VCARD_TEL,  VCARD_TEL_MSG);
	insertTableValue(phonenumbers, i18n("Mobile"), VCARD_TEL,  VCARD_TEL_CELL);
	insertTableValue(phonenumbers, i18n("Pager"), VCARD_TEL,  VCARD_TEL_PAGER);
	insertTableValue(phonenumbers, i18n("Bulletin Board System"), VCARD_TEL,  VCARD_TEL_BBS);
	insertTableValue(phonenumbers, i18n("Modem"), VCARD_TEL,  VCARD_TEL_MODEM);
	insertTableValue(phonenumbers, i18n("Car"), VCARD_TEL,  VCARD_TEL_CAR);
	insertTableValue(phonenumbers, i18n("ISDN"), VCARD_TEL,  VCARD_TEL_ISDN);
	insertTableValue(phonenumbers, i18n("Video"), VCARD_TEL,  VCARD_TEL_VIDEO);
	insertTableValue(phonenumbers, i18n("Personal Comm. Services"), VCARD_TEL,  VCARD_TEL_PCS);

	if (!phonenumbers.isEmpty())
	{
		vcardstring += "<b>" + i18n("Phone numbers") + ":</b><br/>\n";
		vcardstring += "<table>\n";
		vcardstring += phonenumbers;
		vcardstring += "</table>\n";
		vcardstring += "<br/>\n";
	}

	// Misc
	QString misc;
	insertTableValue(misc, i18n("Mailer"), VCARD_MAILER);
	insertTableValue(misc, i18n("vCard made by"), VCARD_PRODID);
	insertTableValue(misc, i18n("vCard Version"), VCARD_VERSION);
	insertTableValue(misc, i18n("Last revised"), VCARD_REV);
	insertTableValue(misc, i18n("Class"), VCARD_CLASS);
	insertTableValue(misc, i18n("Role"), VCARD_ROLE);
	insertTableValue(misc, i18n("UID"), VCARD_UID);

	values = m_vcard->getValues(VCARD_GEO);
	if (!values.isEmpty())
	{
		tmpstr.sprintf("(%s, %s)", values[0].ascii(), values[1].ascii());
		misc += "<tr><td>" + i18n("Location") + "</td><td>" + tmpstr + "</td></tr>\n";
	}

	values = m_vcard->getValues(VCARD_TZ);
	if (!values.isEmpty())
	{
		tmpstr = "";
		for (unsigned int i = 0; i < values.count(); i++)
		{
			tmpstr += values[i];
			if (i != values.count()-1)
				tmpstr += ", ";
		}
		misc += "<tr><td>" + i18n("Timezone") + "</td><td>" + tmpstr + "</td></tr>\n";
	}

	if (!misc.isEmpty())
	{
		vcardstring += "<b>" + i18n("Miscellanous") + ":</b><br/>\n";
		vcardstring += "<table>\n";
		vcardstring += misc;
		vcardstring += "</table>\n";
		vcardstring += "<br/><br/>\n";
	}

	// Note
	QString note = m_vcard->getValue(VCARD_NOTE);
	if (!note.isEmpty())
	{
		vcardstring += "<p><b>"+ i18n("Note") + "</b><br/>";
		vcardstring += note;
		vcardstring += "</p>\n";
	}

	vcardstring += "</qt>";
	for (i=0; vcardstring.contains("\t"); i++)
		vcardstring.remove(vcardstring.find("\t"), 1);

	kdDebug() << vcardstring << endl;
	m_view->setText(vcardstring);
}



bool KvcardLayout::insertTableValues(QString& output, const char* label, const char* key, const char* qualified_key)
{
	QValueList<QString> values;
	QString tmpstr;
	bool first_content=true;

	if (qualified_key==0)
		values =  m_vcard->getValues(key);
	else
		values = m_vcard->getValues(key, qualified_key);
	
	if (values.count()==0)
		return false;
	
	QValueList<QString>::ConstIterator  iter;
	for (iter=values.begin(); iter != values.end(); iter++)
	{
		(*iter).stripWhiteSpace();
		if (!(*iter).isEmpty())
		{
			if (!first_content)
				tmpstr += "<br/>";
			tmpstr +=  *iter ;
			first_content=false;
		}
	}
	if (!tmpstr.isEmpty())
	{
		output += "<tr><td><i>" +  QString(label) +  "</i></td>";
		output += "<td>" + tmpstr + "</td></tr>\n";
		return true;
	}
	return false;
}

bool KvcardLayout::insertTableValue(QString& output, const char* label, const char* key, const char* qualified_key)
{
	QString value;
	if (qualified_key==0)
		value =   m_vcard->getValue(key);
	else
		value = m_vcard->getValue(key, qualified_key);
	if (!value.isEmpty())
	{
		output += "<tr><td><i>" +  QString(label) +  "</i></td>";
		output += "<td>" + value + "</td></tr>\n";
		return true;
	}
	return false;
}

bool KvcardLayout::insertValue(QString& output, const char* label, const char* key, const char* qualified_key)
{
	QString value;
	if (qualified_key==0)
		value =   m_vcard->getValue(key);
	else
		value = m_vcard->getValue(key, qualified_key);
	if (!value.isEmpty())
	{
		output += label;
		output += " : ";
		output += value;
		output += "\n";
		return true;
	}
	return false;
}

bool KvcardLayout::insertAddresses(QString& output)
{
	QValueList<VCardLine> lines;
	QValueList<VCardLine>::Iterator lineIter;

	lines =  m_vcard->getLines(VCARD_ADR);
	if (lines.count()==0)
		return false;
		
	// sort lines: first default (usualy non-existant), then home and work
	/*
	for (lineIter=lines.begin(); lineIter!=lines.end(); lineIter++)
	{
		if ((*lineIter).hasQualifierOrType(VCARD_ADR_HOME))
			lineIter = lines.remove(lineIter);
		if ((*lineIter).hasQualifierOrType(VCARD_ADR_WORK))
			lineIter = lines.remove(lineIter);
	}
	lines += m_vcard->getLines(VCARD_ADR, VCARD_ADR_HOME);
	lines += m_vcard->getLines(VCARD_ADR, VCARD_ADR_WORK);
	*/
		
	for (lineIter=lines.begin(); lineIter!=lines.end(); lineIter++)
	{
		QValueList<QString> values;
		QValueList<QString>::ConstIterator valueIter;
		QString tmpstr;
		bool first_content=true;
		
		values = (*lineIter).getValues();
		for (valueIter=values.begin(); valueIter != values.end(); valueIter++)
		{
			(*valueIter).stripWhiteSpace();
			if (!(*valueIter).isEmpty())
			{
				if (!first_content)
					tmpstr += "<br/>";
				tmpstr +=  *valueIter ;
				kdDebug() << "address value: " << *valueIter << endl;
				first_content=false;
			}
		}	
		
		if (!tmpstr.isEmpty())
		{
			QString label;
			if ( (*lineIter).hasQualifierOrType(VCARD_ADR_HOME))
				label=homestring;
			else if ( (*lineIter).hasQualifierOrType(VCARD_ADR_WORK))
				label=workstring;
			else
				label=defaultstring;
				
			output += "<tr><td><i>" +  label +  "</i>";
			if ((*lineIter).hasQualifierOrType(VCARD_ADR_DOM))
				output += ", " + domstring;
			if ((*lineIter).hasQualifierOrType(VCARD_ADR_INTL))
				output += ", " + intlstring;
			if ((*lineIter).hasQualifierOrType(VCARD_ADR_POSTAL))
				output += ", " + postalstring;
			if ((*lineIter).hasQualifierOrType(VCARD_ADR_PARCEL))
				output += ", " + parcelstring;
			if ((*lineIter).hasQualifierOrType(VCARD_ADR_PREF))
				output += "<br/>" + prefstring;
								
			output += "</td>";
			output += "<td>" + tmpstr + "<br/></td></tr>\n";
		}
	}
	return true;
}


bool KvcardLayout::insertIdentity(QString& output)
{
	QValueList<QString> values;
	QString tmpstr;

	// name
	tmpstr = m_vcard->getValue(VCARD_FN);
	if (tmpstr.length() == 0)
	{
		values = m_vcard->getValues(VCARD_N);
		tmpstr = "";
		if (values.count() >= 4)
		{
			tmpstr += values[3];
			tmpstr += " ";
		}
		if (values.count() >= 2)
		{
			tmpstr += values[1];
			tmpstr += " ";
		}
		if (values.count() >= 3)
		{
			tmpstr += values[2];
			tmpstr += " ";
		}
		tmpstr += values[0];
		if (values.count() >= 5)
		{
			tmpstr += " ";
			tmpstr += values[4];
		}
	}
	QString nn = m_vcard->getValue(VCARD_NICKNAME);
	if (nn != "")
	{
		tmpstr += " (";
		tmpstr += nn;
		tmpstr += ")";
	}
	output += tmpstr;
	return true;
}





