/* This file is part of the KDE project
   Copyright (C) 2000 Eva Brucherseifer <eva@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef KVCARDLAYOUT_H
#define KVCARDLAYOUT_H

#include <qwidget.h>
#include <qvbox.h>
#include <qscrollview.h>
#include <qstring.h>


class VCard;

class QTextView;

/**
  * @class KvcardLayout
  * @author Eva Brucherseifer
  *
  * KvcardLayout is responsible to make the main layout in 
  * the vCard viewer. The output string (qrichtext format)
  * is written to the QTextView Widget.
  * The look of the layout can be changed via KConfig - after 
  * changin values, syncPreferences() must be called to use the
  * new values.
  * 
  */

class KvcardLayout : public QObject
{
	Q_OBJECT
public:
	KvcardLayout(QTextView* view);
	~KvcardLayout();

	void displayVCard(VCard* newcard);
	void syncPreferences();

protected:
	bool insertValue(QString& output, const char* label, const char* key, const char* qualified_key=0);
	bool insertTableValue(QString& output, const char* label, const char* key, const char* qualified_key=0);
	bool insertTableValues(QString& output, const char* label, const char* key, const char* qualified_key=0);
	bool insertIdentity(QString& output);
	bool insertAddresses(QString& output);
	
private:
	VCard* m_vcard;
	QTextView* m_view;
	
	int fontSize;
	bool dontScale;
	QString fontName;
	bool sameFamily;
	QString colorMode;
	QColor backgroundColor;
	QColor foregroundColor;
	QColor linkColor;
	bool sameColor;
};

#endif
